add missing include statement
[project/uci.git] / uci.h
diff --git a/uci.h b/uci.h
index e0b3d30..cfcdf3e 100644 (file)
--- a/uci.h
+++ b/uci.h
@@ -2,9 +2,9 @@
  * libuci - Library for the Unified Configuration Interface
  * Copyright (C) 2008 Felix Fietkau <nbd@openwrt.org>
  *
  * libuci - Library for the Unified Configuration Interface
  * Copyright (C) 2008 Felix Fietkau <nbd@openwrt.org>
  *
- * this program is free software; you can redistribute it and/or modify
- * it under the terms of the gnu lesser general public license version 2.1
- * as published by the free software foundation
+ * This program is free software; you can redistribute it and/or modify
+ * it under the terms of the GNU Lesser General Public License version 2.1
+ * as published by the Free Software Foundation
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -35,6 +35,7 @@
 
 #define UCI_CONFDIR "/etc/config"
 #define UCI_SAVEDIR "/tmp/.uci"
 
 #define UCI_CONFDIR "/etc/config"
 #define UCI_SAVEDIR "/tmp/.uci"
+#define UCI_FILEMODE   0600
 
 enum
 {
 
 enum
 {
@@ -104,6 +105,7 @@ extern void uci_perror(struct uci_context *ctx, const char *str);
  * @single: ignore the 'package' keyword and parse everything into a single package
  *
  * the name parameter is for config files that don't explicitly use the 'package <...>' keyword
  * @single: ignore the 'package' keyword and parse everything into a single package
  *
  * the name parameter is for config files that don't explicitly use the 'package <...>' keyword
+ * if 'package' points to a non-null struct pointer, enable history tracking and merge 
  */
 extern int uci_import(struct uci_context *ctx, FILE *stream, const char *name, struct uci_package **package, bool single);
 
  */
 extern int uci_import(struct uci_context *ctx, FILE *stream, const char *name, struct uci_package **package, bool single);
 
@@ -176,20 +178,30 @@ extern int uci_set_element_value(struct uci_context *ctx, struct uci_element **e
 extern int uci_set(struct uci_context *ctx, struct uci_package *p, char *section, char *option, char *value);
 
 /**
 extern int uci_set(struct uci_context *ctx, struct uci_package *p, char *section, char *option, char *value);
 
 /**
- * uci_del_element: Delete a section or option
+ * uci_rename: Rename an element
+ * @ctx: uci context
+ * @package: package name
+ * @section: section name
+ * @option: option name
+ * @name: new name
+ */
+extern int uci_rename(struct uci_context *ctx, struct uci_package *p, char *section, char *option, char *name);
+
+/**
+ * uci_delete_element: Delete a section or option
  * @ctx: uci context
  * @e: element (section or option)
  */
  * @ctx: uci context
  * @e: element (section or option)
  */
-extern int uci_del_element(struct uci_context *ctx, struct uci_element *e);
+extern int uci_delete_element(struct uci_context *ctx, struct uci_element *e);
 
 /**
 
 /**
* uci_del: Delete a section or option
  * @ctx: uci context
  * @p: uci package
  * @section: section name
  * @option: option name (optional)
  */
  * @ctx: uci context
  * @p: uci package
  * @section: section name
  * @option: option name (optional)
  */
-extern int uci_del(struct uci_context *ctx, struct uci_package *p, char *section, char *option);
+extern int uci_delete(struct uci_context *ctx, struct uci_package *p, char *section, char *option);
 
 /**
  * uci_save: save change history for a package
 
 /**
  * uci_save: save change history for a package
@@ -201,28 +213,76 @@ extern int uci_save(struct uci_context *ctx, struct uci_package *p);
 /**
  * uci_commit: commit changes to a package
  * @ctx: uci context
 /**
  * uci_commit: commit changes to a package
  * @ctx: uci context
- * @p: uci_package struct
+ * @p: uci_package struct pointer
+ * @overwrite: overwrite existing config data and flush history
+ *
+ * committing may reload the whole uci_package data,
+ * the supplied pointer is updated accordingly
  */
  */
-extern int uci_commit(struct uci_context *ctx, struct uci_package *p);
+extern int uci_commit(struct uci_context *ctx, struct uci_package **p, bool overwrite);
 
 /**
  * uci_list_configs: List available uci config files
 
 /**
  * uci_list_configs: List available uci config files
+ * @ctx: uci context
+ */
+extern int uci_list_configs(struct uci_context *ctx, char ***list);
+
+/** 
+ * uci_set_savedir: override the default history save directory
+ * @ctx: uci context
+ * @dir: directory name
+ */
+extern int uci_set_savedir(struct uci_context *ctx, const char *dir);
+
+/** 
+ * uci_set_savedir: override the default config storage directory
+ * @ctx: uci context
+ * @dir: directory name
+ */
+extern int uci_set_confdir(struct uci_context *ctx, const char *dir);
+
+/**
+ * uci_add_history_path: add a directory to the search path for change history files
+ * @ctx: uci context
+ * @dir: directory name
  *
  *
+ * This function allows you to add directories, which contain 'overlays'
+ * for the active config, that will never be committed.
+ */
+extern int uci_add_history_path(struct uci_context *ctx, const char *dir);
+
+/**
+ * uci_revert: revert all changes to a config item
+ * @ctx: uci context
+ * @p: pointer to a uci_package struct ptr (will be replaced by the revert)
+ * @section: section name (optional)
+ * @option option name (optional)
+ */
+extern int uci_revert(struct uci_context *ctx, struct uci_package **p, char *section, char *option);
+
+/**
+ * uci_parse_argument: parse a shell-style argument, with an arbitrary quoting style
  * @ctx: uci context
  * @ctx: uci context
+ * @stream: input stream
+ * @str: pointer to the current line (use NULL for parsing the next line)
+ * @result: pointer for the result
  */
  */
-extern char **uci_list_configs(struct uci_context *ctx);
+extern int uci_parse_argument(struct uci_context *ctx, FILE *stream, char **str, char **result);
 
 /* UCI data structures */
 enum uci_type {
        UCI_TYPE_HISTORY = 0,
        UCI_TYPE_PACKAGE = 1,
        UCI_TYPE_SECTION = 2,
 
 /* UCI data structures */
 enum uci_type {
        UCI_TYPE_HISTORY = 0,
        UCI_TYPE_PACKAGE = 1,
        UCI_TYPE_SECTION = 2,
-       UCI_TYPE_OPTION = 3
+       UCI_TYPE_OPTION = 3,
+       UCI_TYPE_PATH = 4
 };
 
 enum uci_flags {
 };
 
 enum uci_flags {
-       UCI_FLAG_STRICT = (1 << 0), /* strict mode for the parser */
-       UCI_FLAG_PERROR = (1 << 1), /* print error messages to stderr */
+       UCI_FLAG_STRICT =        (1 << 0), /* strict mode for the parser */
+       UCI_FLAG_PERROR =        (1 << 1), /* print parser error messages */
+       UCI_FLAG_EXPORT_NAME =   (1 << 2), /* when exporting, name unnamed sections */
+       UCI_FLAG_SAVED_HISTORY = (1 << 3), /* store the saved history in memory as well */
 };
 
 struct uci_element
 };
 
 struct uci_element
@@ -243,6 +303,12 @@ struct uci_context
        /* uci runtime flags */
        enum uci_flags flags;
 
        /* uci runtime flags */
        enum uci_flags flags;
 
+       char *confdir;
+       char *savedir;
+
+       /* search path for history files */
+       struct uci_list history_path;
+
        /* private: */
        int errno;
        const char *func;
        /* private: */
        int errno;
        const char *func;
@@ -252,22 +318,6 @@ struct uci_context
        int bufsz;
 };
 
        int bufsz;
 };
 
-struct uci_parse_context
-{
-       /* error context */
-       const char *reason;
-       int line;
-       int byte;
-
-       /* private: */
-       struct uci_package *package;
-       struct uci_section *section;
-       FILE *file;
-       const char *name;
-       char *buf;
-       int bufsz;
-};
-
 struct uci_package
 {
        struct uci_element e;
 struct uci_package
 {
        struct uci_element e;
@@ -279,6 +329,7 @@ struct uci_package
        /* private: */
        int n_section;
        struct uci_list history;
        /* private: */
        int n_section;
        struct uci_list history;
+       struct uci_list saved_history;
 };
 
 struct uci_section
 };
 
 struct uci_section
@@ -286,6 +337,7 @@ struct uci_section
        struct uci_element e;
        struct uci_list options;
        struct uci_package *package;
        struct uci_element e;
        struct uci_list options;
        struct uci_package *package;
+       bool anonymous;
        char *type;
 };
 
        char *type;
 };
 
@@ -299,7 +351,8 @@ struct uci_option
 enum uci_command {
        UCI_CMD_ADD,
        UCI_CMD_REMOVE,
 enum uci_command {
        UCI_CMD_ADD,
        UCI_CMD_REMOVE,
-       UCI_CMD_CHANGE
+       UCI_CMD_CHANGE,
+       UCI_CMD_RENAME
 };
 
 struct uci_history
@@ -176,20 +178,30 @@ extern int uci_set_element_value(struct uci_context *ctx, struct uci_element **e
 
 /**
- * uci_del: Delete a section or option
+ * uci_delete: Delete a section or option
  * @ctx: uci context
  * @p: uci package
 };
 
 struct uci_history